#if !defined  HAVE_BIT_PAPER_FOLD_H__
#define       HAVE_BIT_PAPER_FOLD_H__


#include "bits/bitcount.h"

#include "fxttypes.h"  // ulong


static inline bool bit_paper_fold(ulong k)
// Return element number k of the paper-folding sequence:
// k=      0, 1, 2, 3, 4, 5, ...
// seq(k)= 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, ...
// Sequence (for k>=1) is entry A014577 of the OEIS.
// Also: for k>=1, the dragon curve turns left if seq(k)=1,
//   else turn right.
{
    ulong h = k & -k;  // == lowest_one(k)
    k &= (h<<1);
    return  ( k==0 );
}
// -------------------------

static inline ulong bit_dragon_rot(ulong k)
// Return total rotation (as multiple of the right angle)
//   after k steps in the dragon curve.
// k=      0, 1, 2, 3, 4, 5, ...
// seq(k)= 0, 1, 2, 1, 2, 3, 2, 1, 2, 3, 4, 3, 2, 3, 2, ...
// move =  +  ^  -  ^  -  v  -  ^  -  v  +  v  -  v  -  ...
// (0==start, +==right, -==left, ^==up, v==down).
// Sequence is entry A005811 of the OEIS.
// Take result modulo 4 to ignore multiples of 360 degree.
// Algorithm: count the ones in gray_code(k).
{
    return bit_count( k ^ (k>>1) );
}
// -------------------------


#endif  // !defined HAVE_BIT_PAPER_FOLD_H__
